// Backlight.cpp: implementation of the Backlight class.
//
// ADS 710020-11551
//
//////////////////////////////////////////////////////////////////////

#include <Windows.h>
#include "Backlight.h"

#define BACKLIGHT_BASE		(0xF0000000)
#define BACKLIGHT_OFFSET	(0x0004)
#define BACKLIGHT_BIT		(0x00000008)

#define BACKLIGHT_REGISTER  ((void *)(BACKLIGHT_BASE + BACKLIGHT_OFFSET))

// Register Macros
#define READ_REG(r)		*((volatile DWORD*)(r))
#define WRITE_REG(r, v) *((volatile DWORD*)(r)) = (v)

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Backlight::Backlight(int frequency, BOOL isActiveHigh)
{
	reg = NULL;

	reg = (volatile DWORD *) MapPhysicalAddr(BACKLIGHT_REGISTER, sizeof(DWORD));

	if(reg == NULL)
	{
		RETAILMSG(TRUE,(L"Error:  Unable to map physical register address\r\n"));
	}

	if ((brightnessEvent = CreateEvent(NULL, FALSE, FALSE, BACKLIGHT_ATTRIBUTES_EVENT_NAME)) == NULL)
	{
		RETAILMSG(TRUE,(L"Error:  Unable to create event\r\n"));
	}

	if (frequency != 0)
    {
		attributes.fields.wReserved    = (WORD) frequency;
        attributes.fields.Rsvd1        = ((isActiveHigh) ? 1 : 0);
        attributes.fields.Rsvd2        = 1;
    }
    else
    {
		attributes.fields.Rsvd2        = 0;
    }
}

void Backlight::On()
{
	WRITE_REG(reg, READ_REG(reg) & ~BACKLIGHT_BIT);
}

void Backlight::Off()
{
	WRITE_REG(reg, READ_REG(reg) | BACKLIGHT_BIT);
}

void Backlight::SetLevel(BYTE level)
{
	attributes.fields.bLevel = level;
    
	if (SetEventData(brightnessEvent, attributes.dwData))
    {
        if (!SetEvent(brightnessEvent))
        {
           RETAILMSG(TRUE,(L"Error:  Unable to set event: #%d\r\n", GetLastError()));
        }
    }
    else
    {
        RETAILMSG(TRUE,(L"Error:  Unable to set event data: #%d\r\n", GetLastError()));
    }
}

Backlight::~Backlight()
{
	VirtualFree((PVOID)reg,0,MEM_RELEASE);
	CloseHandle(brightnessEvent);
}

// This function maps Physical address to Virtual Address
//-------------------------------------------------------
#define PAGE_SIZE 				0x1000 	// 1 page:4KB
#define PAGE_ALIGN(Vaddr) 		((PVOID)((ULONG)(Vaddr) & ~(PAGE_SIZE - 1)))
#define ROUND_TO_PAGES(Size)  	(((ULONG)(Size) + PAGE_SIZE - 1) & ~(PAGE_SIZE - 1))

PVOID MapPhysicalAddr(PVOID dwPhysicalAddress, DWORD dwSize)
{
	LPVOID pVirtualAddr;
	PVOID PageAlinedPhysicalAddress;
	ULONG Offset;
	DWORD PageAlignedSize;
	BOOL bCopyReturn;

	// Get Page Aligned Address
	PageAlinedPhysicalAddress = PAGE_ALIGN (dwPhysicalAddress);
	Offset = (ULONG)dwPhysicalAddress - (ULONG)PageAlinedPhysicalAddress;

	// Get Page Aligned Size
	PageAlignedSize = ROUND_TO_PAGES(dwSize + Offset);

	// Create reserved block of virtual memory
	pVirtualAddr = VirtualAlloc(0, PageAlignedSize, MEM_RESERVE, PAGE_NOACCESS);

    if(pVirtualAddr == NULL) 
	{
		RETAILMSG(1,(L"	MapPhysicalAddr : Error Return. \r\n"));
        return NULL;
    }

	// Map physical memory to virtual memory
 	// then /256 and use the PAGE_PHYSICAL flag
	//----------------------------------------------
    bCopyReturn = VirtualCopy (	pVirtualAddr, 
								(LPVOID)((ULONG)PageAlinedPhysicalAddress >> 8),
								PageAlignedSize, 
								PAGE_READWRITE | PAGE_NOCACHE | PAGE_PHYSICAL);

    if(bCopyReturn == FALSE) 
	{
		RETAILMSG(1,(L"	MapPhysicalAddr : return false from VirtualCopy(). \r\n"));      
        VirtualFree( pVirtualAddr, 0, MEM_RELEASE  );
        return NULL;
    }

	pVirtualAddr = (PVOID)((PUCHAR)pVirtualAddr + Offset);

	return pVirtualAddr;
}