///////////////////////////////////////////////////
// Backlight.h
// 710020-11551
//
// Applied Data Systems
//
// Description
// -----------
// Interface for accessing the Backlight funtions on the Sphere Board
// 
// Requirements
// ------------
// Needs the Backlight driver built into the OS image with the properly configured
// event name defined by BACKLIGHT_ATTRIBUTES_EVENT_NAME
//
// History
// -------
// 710020-11551: August 23, 2006 (jnosek)
//  - Initial implementation
///////////////////////////////////////////////////

#include <windows.h>

#define BACKLIGHT_ATTRIBUTES_EVENT_NAME TEXT("Display/BackLight/AttributesEvent")

//////////////////////////////////////////////////////////////////////
// Stucts and Unions
//////////////////////////////////////////////////////////////////////
typedef struct
{
    WORD wReserved;
    BYTE bLevel;
    BYTE Rsvd1:1;
    BYTE Rsvd2:1;
    BYTE Rsvd3:1;
    BYTE Rsvd4:1;
    BYTE Rsvd5:1;
    BYTE Rsvd6:1;
    BYTE Rsvd7:1;
    BYTE Rsvd8:1;
}
    BL_ATTR_STRUCT;


typedef union
{
    BL_ATTR_STRUCT  fields;
    DWORD           dwData;
}
    BL_ATTR_UNION;


////////////////////////////////////////////////////////////////////////
// Extern and Global Declerations
////////////////////////////////////////////////////////////////////////
extern "C" __declspec(dllimport) BOOL VirtualCopy(LPVOID lpvDest, LPVOID lpvSrc, DWORD cbSize, DWORD fdwProtect);

extern "C" __declspec(dllimport) BOOL SetEventData(HANDLE hEvent,DWORD dwData); 

PVOID MapPhysicalAddr(PVOID dwPhysicalAddress, DWORD dwSize);


///////////////////////////////////////////////////////////////////////
// Backlight Class Definition
///////////////////////////////////////////////////////////////////////

class Backlight
{
private:
	volatile		DWORD* reg;
	HANDLE			brightnessEvent;
	BL_ATTR_UNION   attributes;

public:
	// Backlight Class constructor
	// Parameters:
	// frequency:    The frequency of the PWM used to control the backlight, recommneded setting is 255
	// isActiveHigh: If the value is true, the display is active high, if it is false, the display is active low
	Backlight(int frequency, BOOL isActiveHigh);
	
	// Deconstructor
	~Backlight();

	// Turns the backlight ON
	void On();
	
	// Turns the backlight OFF
	void Off();
	
	// Sets the level of the backlight
	void SetLevel(BYTE level);
};

